@@ -14,12 +14,5 @@ if test "$1" = "--build-w32"; then
     exit
 fi
 
-aclocal -I . && \
-libtoolize -c --force --automake && \
-aclocal -I . && \
-autoheader  && \
-automake -a -c --gnu || automake -a -c --gnu
-autoconf
-
-
-echo "You can now run \"./configure --enable-maintainer-mode\" and \"make\""
+autoreconf --force --install
+echo "You can now run \"./configure && make\""
