@@ -1,6 +1,5 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * sig-check.c - Check signatures
- *        Copyright (C) 2001, 2002, 2003 Timo Schulz
+/* sig-check.c - Check signatures
+ *        Copyright (C) 2001, 2002, 2003, 2004 Timo Schulz
  *        Copyright (C) 1998,1999,2000,2001,2002 Free Software Foundation, Inc.
  *
  * This file is part of OpenCDK.
@@ -17,7 +16,7 @@
  *  
  * You should have received a copy of the GNU General Public License 
  * along with OpenCDK; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,8 +39,10 @@ hash_mpibuf( cdk_pkt_pubkey_t pk, cdk_md_hd_t md )
     npkey = cdk_pk_get_npkey( pk->pubkey_algo );
     for( i = 0; i < npkey; i++ ) {
         a = pk->mpi[i];
-        if( pk->version == 4 )
-            cdk_md_write( md, a->data, 2 );
+        if( pk->version == 4 ) {
+            cdk_md_putc( md, a->bits >> 8 );
+            cdk_md_putc( md, a->bits );
+        }
         cdk_md_write( md, a->data + 2, a->bytes );
     }
 }
@@ -50,7 +51,7 @@ hash_mpibuf( cdk_pkt_pubkey_t pk, cdk_md_hd_t md )
 void
 _cdk_hash_pubkey( cdk_pkt_pubkey_t pk, cdk_md_hd_t md, int usefpr )
 {
-    byte buf[12];
+    byte buf[4];
     u16 n;
     int i, npkey;
 
@@ -67,26 +68,26 @@ _cdk_hash_pubkey( cdk_pkt_pubkey_t pk, cdk_md_hd_t md, int usefpr )
         n += pk->mpi[i]->bytes;
         n += 2;
     }
-
-    i = 0;
-    buf[i++] = 0x99;
-    buf[i++] = n >> 8;
-    buf[i++] = n;
-    buf[i++] = pk->version;
-    buf[i++] = pk->timestamp >> 24;
-    buf[i++] = pk->timestamp >> 16;
-    buf[i++] = pk->timestamp >> 8;
-    buf[i++] = pk->timestamp;
+  
+    cdk_md_putc( md, 0x99 );
+    cdk_md_putc( md, n >> 8 );
+    cdk_md_putc( md, n );
+    cdk_md_putc( md, pk->version );
+  
+    buf[0] = pk->timestamp >> 24;
+    buf[1] = pk->timestamp >> 16;
+    buf[2] = pk->timestamp >> 8;
+    buf[3] = pk->timestamp;
+    cdk_md_write( md, buf, 4 );
 
     if( pk->version < 4 ) {
         u16 a = 0;
         if( pk->expiredate )
             a = (u16)((pk->expiredate - pk->timestamp) / 86400L);
-        buf[i++] = a >> 8;
-        buf[i++] = a;
+        cdk_md_putc( md, a >> 8 );
+        cdk_md_putc( md, a );
     }
-    buf[i++] = pk->pubkey_algo;
-    cdk_md_write( md, buf, i );
+    cdk_md_putc( md, pk->pubkey_algo );
     hash_mpibuf( pk, md );
 }
 
@@ -127,58 +128,43 @@ _cdk_hash_userid( cdk_pkt_userid_t uid, int is_v4, cdk_md_hd_t md )
 void
 _cdk_hash_sig_data( cdk_pkt_signature_t sig, cdk_md_hd_t md )
 {
-    byte * buf;
+    byte buf[4];
     size_t n = 0;
-    int i = 0;
 
     if( !sig || !md )
         return;
 
-    buf = cdk_calloc( 1, 32 );
-    if( !buf )
-        return;
-    
     if( sig->version == 4 )
-        buf[i++] = sig->version;
-    buf[i++] = sig->sig_class;
+        cdk_md_putc( md, sig->version );
+    cdk_md_putc( md, sig->sig_class );
     if( sig->version < 4 ) {
-        buf[i++] = sig->timestamp >> 24;
-        buf[i++] = sig->timestamp >> 16;
-        buf[i++] = sig->timestamp >> 8;
-        buf[i++] = sig->timestamp;
+        buf[0] = sig->timestamp >> 24;
+        buf[1] = sig->timestamp >> 16;
+        buf[2] = sig->timestamp >> 8;
+        buf[3] = sig->timestamp;
+        cdk_md_write( md, buf, 4 );
     }
     else {
-        buf[i++] = sig->pubkey_algo;
-        buf[i++] = sig->digest_algo;
+        cdk_md_putc( md, sig->pubkey_algo );
+        cdk_md_putc( md, sig->digest_algo );
         if( sig->hashed ) {
-            byte * p = _cdk_subpkt_get_array( sig->hashed, 0, &n );
-            if( !p )
-                return;
-            buf = cdk_realloc( buf, i+n+2+1 );
-            if( !buf )
-                return;
-            buf[i++] = n >> 8;
-            buf[i++] = n;
-            memcpy( buf+i, p, n );
-            i += n;
-            cdk_free( p );
+            _cdk_subpkt_hash( sig->hashed, &n, md );
             sig->hashed_size = n;
             n = sig->hashed_size + 6;
 	}
         else {
-            buf[i++] = 0x00;
-            buf[i++] = 0x00;
-            sig->hashed_size = 6;
+            cdk_md_putc( md, 0 );
+            cdk_md_putc( md, 0 );
             n = 6;
 	}
-        buf[i++] = sig->version;
-        buf[i++] = 0xFF;
-        buf[i++] = n >> 24;
-        buf[i++] = n >> 16;
-        buf[i++] = n >> 8;
-        buf[i++] = n;
+        cdk_md_putc( md, sig->version );
+        cdk_md_putc( md, 0xff );
+        buf[0] = n >> 24;
+        buf[1] = n >> 16;
+        buf[2] = n >> 8;
+        buf[3] = n;
+        cdk_md_write( md, buf, 4 );
     }
-    cdk_md_write( md, buf, i );
 }
 
 
@@ -215,8 +201,10 @@ _cdk_sig_check( cdk_pkt_pubkey_t pk, cdk_pkt_signature_t sig,
     if( sig->flags.checked )
         return sig->flags.valid ? 0 : CDK_Bad_Sig;
 
-    if( !KEY_CAN_SIGN( pk->pubkey_algo ) )
+    if( !KEY_CAN_SIGN( pk->pubkey_algo ) ) {
+        _cdk_log_debug ("pk algo `%d is not useable for signing.\n", pk->pubkey_algo);
         return CDK_Inv_Algo;
+    }
     if( pk->timestamp > sig->timestamp || pk->timestamp > cur_time )
         return CDK_Time_Conflict;
 
@@ -239,7 +227,8 @@ _cdk_sig_check( cdk_pkt_pubkey_t pk, cdk_pkt_signature_t sig,
 
 
 int
-_cdk_pk_check_sig( cdk_keydb_hd_t hd, cdk_kbnode_t knode, cdk_kbnode_t snode )
+_cdk_pk_check_sig (cdk_keydb_hd_t hd, cdk_kbnode_t knode, cdk_kbnode_t snode,
+                   int * is_selfsig)
 {
     cdk_md_hd_t md;
     cdk_pkt_pubkey_t pk = NULL, sig_pk = NULL;
@@ -251,6 +240,8 @@ _cdk_pk_check_sig( cdk_keydb_hd_t hd, cdk_kbnode_t knode, cdk_kbnode_t snode )
     if( !knode || !snode )
         return CDK_Inv_Value;
 
+    if (is_selfsig)
+        *is_selfsig = 0;
     if( knode->pkt->pkttype != CDK_PKT_PUBLIC_KEY
         || snode->pkt->pkttype != CDK_PKT_SIGNATURE )
         return CDK_Inv_Value;
@@ -258,9 +249,9 @@ _cdk_pk_check_sig( cdk_keydb_hd_t hd, cdk_kbnode_t knode, cdk_kbnode_t snode )
     sig = snode->pkt->pkt.signature;
     digest_algo = sig->digest_algo;
 
-    rc = cdk_md_open( &md, digest_algo, 0 );
-    if( rc )
-        return rc;
+    md = cdk_md_open( digest_algo, 0 );
+    if( !md )
+        return CDK_Out_Of_Core;
 
     if( sig->sig_class == 0x20 ) { /* key revocation */
         cdk_kbnode_hash( knode, md, 0, 0, 0 );
@@ -297,18 +288,21 @@ _cdk_pk_check_sig( cdk_keydb_hd_t hd, cdk_kbnode_t knode, cdk_kbnode_t snode )
             goto fail;
         }
         cdk_kbnode_hash( knode, md, 0, 0, 0 );
-        cdk_kbnode_hash( node, md, sig->version==4, 0, 0 );
-        if( pk->keyid[0] == sig->keyid[0] && pk->keyid[1] == sig->keyid[1] )
-            rc = _cdk_sig_check( pk, sig, md, &is_expired );
-        else if( hd ) {
-            rc = cdk_keydb_get_pk( hd, sig->keyid, &sig_pk );
+        cdk_kbnode_hash( node, md, sig->version == 4, 0, 0 );
+        if (pk->keyid[0] == sig->keyid[0] && pk->keyid[1] == sig->keyid[1]) {
+            rc = _cdk_sig_check (pk, sig, md, &is_expired);
+            if (is_selfsig)
+                *is_selfsig = 1;
+        }
+        else if (hd) {
+            rc = cdk_keydb_get_pk (hd, sig->keyid, &sig_pk);
             if( !rc )
-                rc = _cdk_sig_check( sig_pk, sig, md, &is_expired );
-            _cdk_free_pubkey( sig_pk );
+                rc = _cdk_sig_check (sig_pk, sig, md, &is_expired);
+            _cdk_free_pubkey (sig_pk);
 	}
     }
  fail:
-    cdk_md_close( md );
+    cdk_md_close (md);
     return rc;
 }
 
@@ -327,46 +321,54 @@ cdk_error_t
 cdk_pk_check_sigs( cdk_kbnode_t knode, cdk_keydb_hd_t hd, int * r_status )
 {
     cdk_pkt_signature_t sig = NULL;
-    cdk_kbnode_t node;
+    cdk_kbnode_t k;
     u32 keyid = 0;
-    int key_status = 0;
+    int key_status = 0, is_selfsig = 0;
+    int no_signer = 0, n_sigs = 0;
     int rc = 0;
 
     if( !knode || !r_status )
         return CDK_Inv_Value;
 
-    node = cdk_kbnode_find( knode, CDK_PKT_PUBLIC_KEY );
-    if( !node )
+    k = cdk_kbnode_find( knode, CDK_PKT_PUBLIC_KEY );
+    if( !k )
         return CDK_Error_No_Key;
-    if( node->pkt->pkt.public_key->is_revoked )
+    if( k->pkt->pkt.public_key->is_revoked )
         key_status |= CDK_KEY_REVOKED;
-    if( node->pkt->pkt.public_key->has_expired )
+    if( k->pkt->pkt.public_key->has_expired )
         key_status |= CDK_KEY_EXPIRED;
     if( key_status ) {
         *r_status = key_status;
         return CDK_General_Error;
     }
-    keyid = cdk_pk_get_keyid( node->pkt->pkt.public_key, NULL );
+    keyid = cdk_pk_get_keyid( k->pkt->pkt.public_key, NULL );
 
-    for( node = knode; node; node = node->next ) {
-        if( node->pkt->pkttype != CDK_PKT_SIGNATURE )
-            continue;
-        sig = node->pkt->pkt.signature;
-        rc = _cdk_pk_check_sig( hd, knode, node );
-        if( rc && IS_UID_SIG( sig ) && rc == CDK_Error_No_Key ) {
-            sig->flags.missing_key = 1;
+    for( k = knode; k && k->pkt->pkttype; k = k->next ) {
+        if( k->pkt->pkttype != CDK_PKT_SIGNATURE )
             continue;
+        sig = k->pkt->pkt.signature;
+        rc = _cdk_pk_check_sig (hd, knode, k, &is_selfsig);
+        if (IS_UID_SIG (sig)) {
+            if (is_selfsig == 0)
+                n_sigs++;
+            if (rc && rc == CDK_Error_No_Key) {
+                sig->flags.missing_key = 1;
+                no_signer++;
+                continue;
+            }
         }
-        else if( rc && rc != CDK_Error_No_Key ) {
+        else if (rc && rc != CDK_Error_No_Key) {
             *r_status = CDK_KEY_INVALID;
+            rc = 0;
             break; /* invalid self signature or key signature */
         }
         _cdk_log_debug( "signature %s: signer %08lX keyid %08lX\n",
                         rc==CDK_Bad_Sig? "BAD" : "good", sig->keyid[1],
                         keyid );
     }
+    if (n_sigs == no_signer)
+        *r_status |= CDK_KEY_NOSIGNER;
     if( !rc || rc == CDK_Error_No_Key )
-        *r_status = CDK_KEY_VALID;
+        *r_status |= CDK_KEY_VALID;
     return rc;
 }
-
