@@ -19,9 +19,9 @@
 
 CC=gcc
 CFLAGS="-O2 -Wall"
-CFLAGS="$CFLAGS -I../src"
+CFLAGS="$CFLAGS -I../src -I/usr/local/include"
 POPT="/usr/include/popt.h"
-LIBS="../src/.libs/libopencdk.a -L/usr/lib -lpopt -L/usr/local/lib -lz -lgcrypt"
+LIBS="../src/.libs/libopencdk.a -lpopt -L/usr/local/lib -lz -lgpg-error -lgcrypt"
 
 echo "check for libpopt"
 if [ -e $POPT ]; then
